@@ -52,6 +52,7 @@ static std::unordered_map<DWORD, bool> expectedErrors = {
   { ERROR_BAD_NET_NAME, false }, // 67
   { ERROR_INVALID_NAME, false }, // 123
   { ERROR_BAD_PATHNAME, false }, // 161
+  { ERROR_CANT_ACCESS_FILE, false }, // 1920
 };
 
 bool File::Exists(const PathName& path, FileExistsOptionSet options)
@@ -378,6 +379,11 @@ void File::Copy(const PathName& source, const PathName& dest, FileCopyOptionSet
       SetNativeAttributes(dest, destAttributes2);
     }
   }
+  PathName dir = dest.GetDirectoryName();
+  if (!dir.Empty() && !Directory::Exists(dir))
+  {
+    Directory::Create(dir);
+  }
   if (!CopyFileW(source.ToExtendedLengthPathName().ToWideCharString().c_str(), dest.ToExtendedLengthPathName().ToWideCharString().c_str(), options[FileCopyOption::ReplaceExisting] ? FALSE : TRUE))
   {
     MIKTEX_FATAL_WINDOWS_ERROR_2("CopyFileW", "existing", source.ToString(), "path", dest.ToString());
