@@ -25,6 +25,11 @@
 #  include <unistd.h>
 #endif
 
+#if defined(MIKTEX_MACOS_BUNDLE)
+#  include <crt_externs.h>
+#  define environ (*_NSGetEnviron ())
+#endif
+
 #include <fstream>
 #include <iostream>
 
@@ -34,6 +39,7 @@
 // FIXME: must be the first MiKTeX header
 #include "core-version.h"
 
+#include <miktex/Core/AutoResource>
 #include <miktex/Core/ConfigNames>
 #include <miktex/Core/Directory>
 #include <miktex/Core/DirectoryLister>
@@ -253,8 +259,6 @@ void SessionImpl::Initialize(const Session::InitInfo& initInfo)
     }
   }
 
-  SetEnvironmentVariables();
-
   trace_core->WriteLine("core", fmt::format(T_("initializing MiKTeX Core version {0}"), MIKTEX_COMPONENT_VERSION_STR));
 
 #if defined(MIKTEX_WINDOWS) && defined(MIKTEX_CORE_SHARED)
@@ -931,13 +935,61 @@ void SessionImpl::Reset()
   this->onFinishScript = move(onFinishScript);
 }
 
-void SessionImpl::SetEnvironmentVariables()
+unordered_map<string, string> SessionImpl::CreateChildEnvironment(bool changeDirectory)
 {
-#if MIKTEX_WINDOWS
-  Utils::SetEnvironmentString("TEXSYSTEM", "miktex");
+  unordered_map<string, string> envMap;
 
-  // Ghostscript
-  Utils::SetEnvironmentString("GSC", MIKTEX_GS_EXE);
+#if defined(MIKTEX_WINDOWS)
+  auto environmentStrings = GetEnvironmentStringsW();
+  MIKTEX_AUTO(FreeEnvironmentStringsW(environmentStrings));
+  for (const wchar_t* env = environmentStrings; *env != 0; ++env)
+  {
+    wstring name;
+    if (*env == L'=')
+    {
+      name += *env++;
+    }
+    for (; *env != L'=' && *env != 0; ++env)
+    {
+      name += *env;
+    }
+    if (*env == L'=')
+    {
+      ++env;
+    }
+    wstring value;
+    for (; *env != 0; ++env)
+    {
+      value += *env;
+    }
+    envMap[WU_(name)] = WU_(value);
+  }
+#else
+  for (char** env = environ; *env != nullptr; ++env)
+  {
+    string name;
+    auto s = *env;
+    for (; *s != '=' && *s != 0; ++s)
+    {
+      name += *s;
+    }
+    if (*s == '=')
+    {
+      ++s;
+    }
+    string value;
+    for (; *s != 0; ++s)
+    {
+      value += *s;
+    }
+    envMap[name] = value;
+  }
+#endif
+
+  envMap["TEXSYSTEM"] = "miktex";
+
+#if defined(MIKTEX_WINDOWS)
+  envMap["GSC"] = MIKTEX_GS_EXE;
 #endif
 
   vector<string> gsDirectories;
@@ -970,12 +1022,12 @@ void SessionImpl::SetEnvironmentVariables()
   MIKTEX_ASSERT(!gsDirectories.Empty());
 
 #if defined(MIKTEX_WINDOWS)
-  Utils::SetEnvironmentString("MIKTEX_GS_LIB", StringUtil::Flatten(gsDirectories, PathNameUtil::PathNameDelimiter));
+  envMap["MIKTEX_GS_LIB"] = StringUtil::Flatten(gsDirectories, PathNameUtil::PathNameDelimiter);
 #else
-  string origGsLib;
-  if (Utils::GetEnvironmentString("GS_LIB", origGsLib))
+  auto it = envMap.find("GS_LIB");
+  if (it != envMap.end())
   {
-    vector<string> origGsLibDirectories = StringUtil::Split(origGsLib, PathNameUtil::PathNameDelimiter);
+    vector<string> origGsLibDirectories = StringUtil::Split(it->second, PathNameUtil::PathNameDelimiter);
     for (const string& d1 : origGsLibDirectories)
     {
       auto it = find_if(gsDirectories.begin(), gsDirectories.end(), [d1](const string& d2) { return PathName::Compare(d1, d2) == 0; });
@@ -985,62 +1037,66 @@ void SessionImpl::SetEnvironmentVariables()
       }
     }
   }
-  Utils::SetEnvironmentString("GS_LIB", StringUtil::Flatten(gsDirectories, PathNameUtil::PathNameDelimiter));
+  envMap["GS_LIB"] = StringUtil::Flatten(gsDirectories, PathNameUtil::PathNameDelimiter);
 #endif
 
-  PathName path = GetTempDirectory();
-
-  if (!HaveEnvironmentString("TEMPDIR") || IsMiKTeXPortable())
+  PathName tempDirectory = GetTempDirectory();
+  for (const string& envName : vector<string>{ "TEMP", "TEMPDIR", "TMP", "TMPDIR" })
   {
-    Utils::SetEnvironmentString("TEMPDIR", path.ToString());
+    auto it = envMap.find(envName);
+    if (it == envMap.end() || !PathName(it->second).IsAbsolute() || IsMiKTeXPortable())
+    {
+      envMap[envName] = tempDirectory.ToString();
+    }
   }
 
-  if (!HaveEnvironmentString("TMPDIR") || IsMiKTeXPortable())
+  for (const string& envName : vector<string>{ "HOME" })
   {
-    Utils::SetEnvironmentString("TMPDIR", path.ToString());
+    auto it = envMap.find(envName);
+    if (it == envMap.end() || !PathName(it->second).IsAbsolute())
+    {
+      envMap[envName] = GetHomeDirectory().ToString();
+    }
   }
 
-  if (!HaveEnvironmentString("TEMP") || IsMiKTeXPortable())
+  vector<string> cwdList;
+  for (const PathName& dir : inputDirectories)
   {
-    Utils::SetEnvironmentString("TEMP", path.ToString());
+    cwdList.push_back(dir.ToString());
   }
-
-  if (!HaveEnvironmentString("TMP") || IsMiKTeXPortable())
+  if (changeDirectory)
   {
-    Utils::SetEnvironmentString("TMP", path.ToString());
+    cwdList.push_back(PathName().SetToCurrentDirectory().ToString());
   }
-
-  if (!HaveEnvironmentString("HOME"))
-  {
-    Utils::SetEnvironmentString("HOME", GetHomeDirectory().ToString());
-  }
-
-  SetCWDEnv();
+  envMap[MIKTEX_ENV_CWD_LIST] = StringUtil::Flatten(cwdList, PathNameUtil::PathNameDelimiter);
 
   if (!initInfo.GetOptions()[InitOption::NoFixPath])
   {
     string envPath;
-    if (!GetEnvironmentString("PATH", envPath))
+    auto it = envMap.find("PATH");
+    if (it != envMap.end())
     {
-      envPath = "";
+      envPath = it->second;
     }
     string newEnvPath;
     bool competition;
     auto p = TryGetBinDirectory(true);
     if (p.first && FixProgramSearchPath(envPath, p.second, false, newEnvPath, competition))
     {
-      Utils::SetEnvironmentString("PATH", newEnvPath);
+      envMap["PATH"] = newEnvPath;
       envPath = newEnvPath;
     }
 #if !defined(MIKTEX_MACOS_BUNDLE)
     p = TryGetBinDirectory(false);
     if (p.first && FixProgramSearchPath(envPath, p.second, false, newEnvPath, competition))
     {
-      Utils::SetEnvironmentString("PATH", newEnvPath);
+      envMap["PATH"] = newEnvPath;
       envPath = newEnvPath;
     }
 #endif
   }
+
+  return envMap;
 }
 
 void SessionImpl::SetTheNameOfTheGame(const string& name)
