@@ -872,7 +872,7 @@ shared_ptr<FileNameDatabase> SessionImpl::GetFileNameDatabase(const char* path)
 
 unsigned SessionImpl::TryDeriveTEXMFRoot(const PathName& path)
 {
-  if (!path.IsFullyQUalified())
+  if (!path.IsFullyQualified())
   {
 #if FIND_FILE_PREFER_RELATIVE_PATH_NAMES
     return INVALID_ROOT_INDEX;
