@@ -1,4 +1,4 @@
-/* 1-1.cpp:
+/* 1-2.cpp:
 
    Copyright (C) 1996-2017 Christian Schenk
 
@@ -23,13 +23,19 @@
 
 #include <miktex/Core/Test>
 
-using namespace MiKTeX::Test;
+#include <string>
 
-BEGIN_TEST_SCRIPT("process-1-1");
+using namespace MiKTeX::Test;
+using namespace std;
+
+BEGIN_TEST_SCRIPT("process-1-2");
 
 BEGIN_TEST_FUNCTION(1);
 {
-  Touch(vecArgs[0].c_str());
+  for (const std::string& arg : vecArgs)
+  {
+    TEST(printf("%s\n", arg.c_str()) == strlen(arg.c_str()) + 1);
+  }
 }
 END_TEST_FUNCTION();
 
