@@ -39,7 +39,6 @@
 #endif
 
 #include <ctime>
-#include <cstdarg>
 
 #include <algorithm>
 #include <codecvt>
@@ -134,9 +133,6 @@ public:
 public:
   void MIKTEXTHISCALL WriteLine(const std::string& facility, const std::string& text) override;
 
-public:
-  void MIKTEXCEECALL WriteFormattedLine(const std::string& facility, const char* format, ...) override;
-
 public:
   TraceStreamImpl(shared_ptr<TraceStreamInfo> info, TraceCallback* callback) :
     info(info),
@@ -197,9 +193,10 @@ void TraceStreamImpl::Logger(const string& facility, TraceLevel level, const str
   }
   for (TraceCallback* callback : info->callbacks)
   {
-    callback->Trace(TraceCallback::TraceMessage(info->name, facility, level, message));
-    // TODO: if(callback->Trace)
-    break;
+    if (callback->Trace(TraceCallback::TraceMessage(info->name, facility, level, message)))
+    {
+      break;
+    }
   }
 }
 
@@ -274,18 +271,6 @@ void TraceStreamImpl::WriteLine(const string& facility, const string& text)
   WriteLine(facility, TraceLevel::Trace, text);
 }
 
-void TraceStreamImpl::WriteFormattedLine(const string& facility, const char* format, ...)
-{
-  if (!IsEnabled(facility, TraceLevel::Trace))
-  {
-    return;
-  }
-  va_list marker;
-  va_start(marker, format);
-  Logger(facility, TraceLevel::Trace, StringUtil::FormatStringVA(format, marker));
-  va_end(marker);
-}
-
 unique_ptr<TraceStream> TraceStream::Open(const string& name, TraceLevel level, TraceCallback* callback)
 {
   lock_guard<mutex> lockGuard(TraceStreamImpl::traceStreamsMutex);
