@@ -23,13 +23,14 @@
 
 #include <errno.h>
 
+#include <WinSock2.h>
+
 #include <miktex/Core/Directory>
 #include <miktex/Core/DirectoryLister>
+#include <miktex/Core/Session>
 #include <miktex/Util/StringUtil>
 #include <miktex/Util/inliners.h>
 
-#include <WinSock2.h>
-
 using namespace MiKTeX::Core;
 using namespace MiKTeX::Util;
 using namespace std;
@@ -111,6 +112,18 @@ MIKTEXUNXCEEAPI(struct dirent*) readdir(DIR* dir)
   C_FUNC_END();
 }
 
+MIKTEXUNXCEEAPI(void) seekdir(DIR* dir, long int loc)
+{
+  // not yet implemented
+  MIKTEX_UNEXPECTED();
+}
+
+MIKTEXUNXCEEAPI(long int) telldir(DIR* dir)
+{
+  // not yet implemented
+  MIKTEX_UNEXPECTED();
+}
+
 MIKTEXUNXCEEAPI(struct wdirent*) wreaddir(WDIR* dir)
 {
   C_FUNC_BEGIN();
@@ -145,9 +158,9 @@ MIKTEXUNXCEEAPI(int) miktex_strncasecmp(const char* lpsz1, const char* lpsz2, si
   return MiKTeX::Util::StringCompare(lpsz1, lpsz2, n, true);
 }
 
-MIKTEXUNXCEEAPI(int) miktex_gettimeofday(struct timeval* ptv, void* pNull)
+MIKTEXUNXCEEAPI(int) miktex_gettimeofday(struct timeval* ptv, void* null)
 {
-  MIKTEX_ASSERT(pNull == nullptr);
+  MIKTEX_ASSERT(null == nullptr);
   MIKTEX_ASSERT(ptv != nullptr);
   SYSTEMTIME systemTime;
   GetSystemTime(&systemTime);
