@@ -22,9 +22,9 @@ struct BezierCurve
   vertexBuffer data;
   double res,res2;
   bool Onscreen;
-  
+
   void init(double res);
-    
+
   // Approximate bounds by bounding box of control polyhedron.
   bool offscreen(size_t n, const triple *v) {
     if(bbox2(n,v).offscreen()) {
@@ -36,35 +36,40 @@ struct BezierCurve
 
   void render(const triple *p, bool straight);
   void render(const triple *p, GLuint I0, GLuint I1);
-  
+
   void append() {
     material1Data.append(data);
   }
-  
+
+  void notRendered() {
+    material1Data.rendered=false;
+  }
+
   void queue(const triple *g, bool straight, double ratio) {
     data.clear();
+    notRendered();
     Onscreen=true;
     init(pixel*ratio);
     render(g,straight);
   }
-  
+
 };
 
 struct Pixel
 {
   vertexBuffer data;
-  
+
   void append() {
     material0Data.append0(data);
   }
-  
+
   void queue(const triple& p, double width) {
     data.clear();
     MaterialIndex=materialIndex;
     data.indices.push_back(data.vertex0(p,width));
     append();
   }
-  
+
   void draw();
 };
 
