@@ -29,6 +29,7 @@
 #include "bbox3.h"
 #include "drawimage.h"
 #include "interact.h"
+#include "fpu.h"
 
 namespace gl {
 #ifdef HAVE_PTHREAD
@@ -84,9 +85,6 @@ vertexBuffer triangleData;
 
 const size_t Nbuffer=10000;
 const size_t nbuffer=1000;
-
-GLuint attributeBuffer;
-GLuint indicesBuffer; 
 }
 
 #endif /* HAVE_GL */
@@ -108,12 +106,15 @@ size_t Nmaterials=1;
 size_t nmaterials=48;
 }
 
+extern void exitHandler(int);
+
 namespace gl {
-  
+
 bool outlinemode=false;
 bool glthread=false;
+bool glupdate=false;
+bool glexit=false;
 bool initialize=true;
-bool exporting=false;
 
 using camp::picture;
 using camp::drawRawImage;
@@ -128,7 +129,8 @@ using settings::Setting;
 
 bool Iconify=false;
 bool ignorezoom;
-int Fitscreen;
+int Fitscreen=1;
+bool firstFit;
 
 bool queueExport=false;
 bool readyAfterExport=false;
@@ -146,8 +148,6 @@ int fullWidth,fullHeight;
 int Width,Height;
 double oWidth,oHeight;
 int screenWidth,screenHeight;
-int maxWidth;
-int maxHeight;
 int maxTileWidth;
 int maxTileHeight;
 
@@ -177,7 +177,7 @@ double *Background;
 size_t Nlights=1; // Maximum number of lights compiled in shader
 size_t nlights; // Actual number of lights
 size_t nlights0;
-triple *Lights; 
+triple *Lights;
 double *Diffuse;
 double *Specular;
 bool antialias;
@@ -205,14 +205,12 @@ mat4 viewMat;
 dmat4 dprojMat;
 dmat4 dprojViewMat;
 dmat4 dviewMat;
-dmat4 drotateMat; 
+dmat4 drotateMat;
 
 const double *dprojView;
 const double *dView;
 double BBT[9];
 
-GLuint ubo;
-
 unsigned int framecount;
 
 template<class T>
@@ -318,7 +316,8 @@ bool Animate;
 bool Step;
 
 #ifdef HAVE_GL
-void idle() 
+
+void idle()
 {
   glutIdleFunc(NULL);
   Xspin=Yspin=Zspin=Animate=Step=false;
@@ -328,21 +327,23 @@ void idle()
 }
 #endif
 
-void home(bool webgl=false) 
+void home(bool webgl=false)
 {
   X=Y=cx=cy=0.0;
-#ifdef HAVE_GL  
+#ifdef HAVE_GL
 #ifdef HAVE_LIBGLUT
-  if(!webgl && !getSetting<bool>("offscreen"))
+#ifndef HAVE_LIBOSMESA
+  if(!webgl)
     idle();
 #endif
+#endif
 #endif
   dviewMat=dmat4(1.0);
   dView=value_ptr(dviewMat);
   viewMat=mat4(dviewMat);
-  
-  drotateMat=dmat4(1.0); 
-  
+
+  drotateMat=dmat4(1.0);
+
   updateModelViewData();
 
   remesh=true;
@@ -390,8 +391,8 @@ GLuint initHDR() {
   std::vector<float> pixels(imspec.width*imspec.height*3);
   imagein->read_image(pixels.data());
 
-  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, imspec.width, imspec.height, 0, 
-    GL_RGB, GL_FLOAT, pixels.data());
+  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, imspec.width, imspec.height, 0,
+               GL_RGB, GL_FLOAT, pixels.data());
 
   glGenerateMipmap(GL_TEXTURE_2D);
   imagein->close();
@@ -400,7 +401,7 @@ GLuint initHDR() {
   return tex;
 }
 
-#endif 
+#endif
 GLint shaderProg,shaderProgColor;
 
 void *glrenderWrapper(void *a);
@@ -457,7 +458,7 @@ void initShaders()
 {
   Nlights=nlights == 0 ? 0 : max(Nlights,nlights);
   Nmaterials=max(Nmaterials,nmaterials);
-  
+
   shaderProg=glCreateProgram();
   string vs=locateFile("shaders/vertex.glsl");
   string fs=locateFile("shaders/fragment.glsl");
@@ -480,7 +481,7 @@ void initShaders()
   shaders.push_back(ShaderfileModePair(fs.c_str(),GL_FRAGMENT_SHADER));
   if(orthographic)
     shaderParams.push_back("ORTHOGRAPHIC");
-    
+
   shaderParams.push_back("WIDTH");
   camp::pixelShader=compileAndLinkShader(shaders,Nlights,Nmaterials,
                                          shaderParams);
@@ -497,7 +498,7 @@ void initShaders()
                                                shaderParams);
 }
 
-void deleteShaders() 
+void deleteShaders()
 {
   glDeleteProgram(camp::transparentShader);
   glDeleteProgram(camp::colorShader);
@@ -507,10 +508,6 @@ void deleteShaders()
 
 void setBuffers()
 {
-  glGenBuffers(1,&camp::attributeBuffer);
-  glGenBuffers(1,&camp::indicesBuffer);
-  glGenBuffers(1,&ubo);
-  
   GLuint vao;
   glGenVertexArrays(1,&vao);
   glBindVertexArray(vao);
@@ -526,14 +523,14 @@ void drawscene(int Width, int Height)
 {
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
   static bool first=true;
-  if(glthread && first && !getSetting<bool>("offscreen")) {
+  if(glthread && first) {
     wait(initSignal,initLock);
     endwait(initSignal,initLock);
     first=false;
   }
 #endif
 
-  if((nlights == 0 && Nlights > 0) || nlights > Nlights || 
+  if((nlights == 0 && Nlights > 0) || nlights > Nlights ||
      nmaterials > Nmaterials) {
     deleteShaders();
     initShaders();
@@ -545,19 +542,19 @@ void drawscene(int Width, int Height)
   triple m(xmin,ymin,zmin);
   triple M(xmax,ymax,zmax);
   double perspective=orthographic ? 0.0 : 1.0/zmax;
-  
+
   double size2=hypot(Width,Height);
-  
+
   if(remesh)
     camp::drawElement::center.clear();
-  
+
   Picture->render(size2,m,M,perspective,remesh);
-  
+
   if(!outlinemode) remesh=false;
 }
 
 // Return x divided by y rounded up to the nearest integer.
-int Quotient(int x, int y) 
+int Quotient(int x, int y)
 {
   return (x+y-1)/y;
 }
@@ -575,8 +572,8 @@ void Export()
       int width=Quotient(fullWidth,Quotient(fullWidth,min(maxTileWidth,Width)));
       int height=Quotient(fullHeight,Quotient(fullHeight,
                                               min(maxTileHeight,Height)));
-      if(settings::verbose > 1) 
-        cout << "Exporting " << Prefix << " as " << fullWidth << "x" 
+      if(settings::verbose > 1)
+        cout << "Exporting " << Prefix << " as " << fullWidth << "x"
              << fullHeight << " image" << " using tiles of size "
              << width << "x" << height << endl;
 
@@ -587,19 +584,18 @@ void Export()
 
       setDimensions(fullWidth,fullHeight,X/Width*fullWidth,Y/Width*fullWidth);
       (orthographic ? trOrtho : trFrustum)(tr,xmin,xmax,ymin,ymax,-zmax,-zmin);
-   
+
       size_t count=0;
-      exporting=true;
       do {
         trBeginTile(tr);
+        fpu_trap(false); // Work around FE_INVALID in OSMesa.
+        remesh=true;
         drawscene(fullWidth,fullHeight);
+        fpu_trap(settings::trap());
         ++count;
       } while (trEndTile(tr));
-      exporting=false;
-
       if(settings::verbose > 1)
-        cout << count << " tile" << (count != 1 ? "s" : "") << " drawn"
-             << endl;
+        cout << count << " tile" << (count != 1 ? "s" : "") << " drawn" << endl;
       trDelete(tr);
 
       picture pic;
@@ -616,24 +612,26 @@ void Export()
       pic.shipout(NULL,Prefix,Format,false,View);
       delete Image;
       delete[] data;
-    } 
+    }
   } catch(handled_error) {
   } catch(std::bad_alloc&) {
     outOfMemory();
   }
+  remesh=true;
   setProjection();
-  bool offscreen=getSetting<bool>("offscreen");
+
+#ifndef HAVE_LIBOSMESA
 #ifdef HAVE_LIBGLUT
-  if(!offscreen)
-    glutPostRedisplay();
+  glutPostRedisplay();
 #endif
 
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
-  if(glthread && readyAfterExport && !offscreen) {
-    readyAfterExport=false;        
+  if(glthread && readyAfterExport) {
+    readyAfterExport=false;
     endwait(readySignal,readyLock);
   }
 #endif
+#endif
 }
 
 void nodisplay()
@@ -656,14 +654,12 @@ inline unsigned int floorpow2(unsigned int n)
   return n-(n >> 1);
 }
 
-void quit() 
+void quit()
 {
 #ifdef HAVE_LIBOSMESA
-  if(getSetting<bool>("offscreen")) {
-    if(osmesa_buffer) delete[] osmesa_buffer;
-    if(ctx) OSMesaDestroyContext(ctx);
-    exit(0);
-  }
+  if(osmesa_buffer) delete[] osmesa_buffer;
+  if(ctx) OSMesaDestroyContext(ctx);
+  exit(0);
 #endif
 #ifdef HAVE_LIBGLUT
   if(glthread) {
@@ -678,7 +674,7 @@ void quit()
       glutDisplayFunc(nodisplay);
       endwait(readySignal,readyLock);
     }
-#endif    
+#endif
     if(interact::interactive)
       glutHideWindow();
 #if defined(MIKTEX)
@@ -690,8 +686,8 @@ void quit()
   }
 #endif
 }
-  
-void mode() 
+
+void mode()
 {
   remesh=true;
   switch(Mode) {
@@ -714,22 +710,23 @@ void mode()
       break;
   }
 #ifdef HAVE_LIBGLUT
-  if(!getSetting<bool>("offscreen"))
-    glutPostRedisplay();
+#ifndef HAVE_LIBOSMESA
+  glutPostRedisplay();
+#endif
 #endif
 }
 
 // GUI-related functions
 #ifdef HAVE_LIBGLUT
-bool capsize(int& width, int& height) 
+bool capsize(int& width, int& height)
 {
   bool resize=false;
-  if(width > maxWidth) {
-    width=maxWidth;
+  if(width > screenWidth) {
+    width=screenWidth;
     resize=true;
   }
-  if(height > maxHeight) {
-    height=maxHeight;
+  if(height > screenHeight) {
+    height=screenHeight;
     resize=true;
   }
   return resize;
@@ -739,14 +736,14 @@ void reshape0(int width, int height)
 {
   X=(X/Width)*width;
   Y=(Y/Height)*height;
-  
+
   Width=width;
   Height=height;
-  
+
   setProjection();
   glViewport(0,0,Width,Height);
 }
-  
+
 void windowposition(int& x, int& y, int width=Width, int height=Height)
 {
   pair z=getSetting<pair>("position");
@@ -765,21 +762,44 @@ void windowposition(int& x, int& y, int width=Width, int height=Height)
 void setsize(int w, int h, bool reposition=true)
 {
   int x,y;
-  
+
   capsize(w,h);
   if(reposition) {
     windowposition(x,y,w,h);
     glutPositionWindow(x,y);
-  }
+  } else
+    glutPositionWindow(max(glutGet(GLUT_WINDOW_X)-2,0),
+                       max(glutGet(GLUT_WINDOW_Y)-2,0));
+
   glutReshapeWindow(w,h);
   reshape0(w,h);
   glutPostRedisplay();
 }
 
-void fullscreen(bool reposition=true) 
+void capzoom()
+{
+  static double maxzoom=sqrt(DBL_MAX);
+  static double minzoom=1.0/maxzoom;
+  if(Zoom <= minzoom) Zoom=minzoom;
+  if(Zoom >= maxzoom) Zoom=maxzoom;
+
+  if(Zoom != lastzoom) remesh=true;
+  lastzoom=Zoom;
+}
+
+void fullscreen(bool reposition=true)
 {
   Width=screenWidth;
   Height=screenHeight;
+  if(firstFit) {
+    if(Width < Height*Aspect)
+      Zoom *= Width/(Height*Aspect);
+    capzoom();
+    setProjection();
+    firstFit=false;
+  }
+  Xfactor=((double) screenHeight)/Height;
+  Yfactor=((double) screenWidth)/Width;
   reshape0(Width,Height);
   if(reposition)
     glutPositionWindow(0,0);
@@ -787,45 +807,46 @@ void fullscreen(bool reposition=true)
   glutPostRedisplay();
 }
 
-void fitscreen(bool reposition=true) 
+void fitscreen(bool reposition=true)
 {
   if(Animate && Fitscreen == 2) Fitscreen=0;
   switch(Fitscreen) {
     case 0: // Original size
     {
       Xfactor=Yfactor=1.0;
-      setsize(oldWidth,oldHeight,reposition);
+      double pixelRatio=getSetting<double>("devicepixelratio");
+      setsize(oldWidth*pixelRatio,oldHeight*pixelRatio,reposition);
       break;
     }
     case 1: // Fit to screen in one dimension
-    {       
+    {
       oldWidth=Width;
       oldHeight=Height;
       int w=screenWidth;
       int h=screenHeight;
-      if(w >= h*Aspect) w=(int) (h*Aspect+0.5);
-      else h=(int) (w/Aspect+0.5);
+      if(w > h*Aspect)
+        w=min((int) ceil(h*Aspect),w);
+      else
+        h=min((int) ceil(w/Aspect),h);
       setsize(w,h,reposition);
       break;
     }
     case 2: // Full screen
     {
-      Xfactor=((double) screenHeight)/Height;
-      Yfactor=((double) screenWidth)/Width;
       fullscreen(reposition);
       break;
     }
   }
 }
 
-void togglefitscreen() 
+void togglefitscreen()
 {
   ++Fitscreen;
   if(Fitscreen > 2) Fitscreen=0;
   fitscreen();
 }
 
-void initTimer() 
+void initTimer()
 {
   gettimeofday(&lasttime,NULL);
   gettimeofday(&lastframetime,NULL);
@@ -843,11 +864,11 @@ void screen()
     fitscreen(false);
 }
 
-void nextframe(int) 
+void nextframe(int)
 {
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
   endwait(readySignal,readyLock);
-#endif    
+#endif
   double framedelay=getSetting<double>("framedelay");
   if(framedelay > 0)
     usleep((unsigned int) (1000.0*framedelay+0.5));
@@ -861,7 +882,7 @@ void display()
     queueScreen=false;
   }
 
-  bool fps=settings::verbose > 2;  
+  bool fps=settings::verbose > 2;
   drawscene(Width,Height);
   if(fps) {
     if(framecount < 10) // Measure steady-state framerate
@@ -871,7 +892,7 @@ void display()
       if(s > 0.0) {
         double rate=1.0/s;
         S.add(rate);
-        cout << "FPS=" << rate << "\t" << S.mean() << " +/- " << S.stdev() 
+        cout << "FPS=" << rate << "\t" << S.mean() << " +/- " << S.stdev()
              << endl;
       }
     }
@@ -899,7 +920,7 @@ void display()
   if(queueExport) {
     Export();
     queueExport=false;
-  } 
+  }
   if(!glthread) {
 #if defined(MIKTEX_WINDOWS)
     // MIKTEX-UNEXPECTED
@@ -912,7 +933,7 @@ void display()
   }
 }
 
-void update() 
+void update()
 {
   glutDisplayFunc(display);
   Animate=getSetting<bool>("autoplay");
@@ -921,7 +942,7 @@ void update()
 
   lastzoom=Zoom;
   double cz=0.5*(zmin+zmax);
-  
+
   dviewMat=translate(translate(dmat4(1.0),dvec3(cx,cy,cz))*drotateMat,
                      dvec3(0,0,-cz));
   dView=value_ptr(dviewMat);
@@ -929,20 +950,34 @@ void update()
 
   setProjection();
   updateModelViewData();
-  
+
   glutPostRedisplay();
 }
 
 void updateHandler(int)
 {
   queueScreen=true;
+  remesh=true;
   update();
   if(interact::interactive || !Animate) {
     glutShowWindow();
   }
 }
 
-void animate() 
+void poll(int)
+{
+  if(glupdate) {
+    updateHandler(0);
+    glupdate=false;
+  }
+  if(glexit) {
+    exitHandler(0);
+    glexit=false;
+  }
+  glutTimerFunc(100.0,poll,0);
+}
+
+void animate()
 {
   Animate=!Animate;
   if(Animate) {
@@ -967,14 +1002,14 @@ void reshape(int width, int height)
 #endif
     }
   }
-  
+
   if(capsize(width,height))
     glutReshapeWindow(width,height);
- 
+
   reshape0(width,height);
   remesh=true;
 }
-  
+
 void shift(int x, int y)
 {
   double Zoominv=1.0/Zoom;
@@ -983,7 +1018,7 @@ void shift(int x, int y)
   x0=x; y0=y;
   update();
 }
-  
+
 void pan(int x, int y)
 {
   if(orthographic) {
@@ -997,17 +1032,6 @@ void pan(int x, int y)
   x0=x; y0=y;
   update();
 }
-  
-void capzoom() 
-{
-  static double maxzoom=sqrt(DBL_MAX);
-  static double minzoom=1.0/maxzoom;
-  if(Zoom <= minzoom) Zoom=minzoom;
-  if(Zoom >= maxzoom) Zoom=maxzoom;
-  
-  if(Zoom != lastzoom) remesh=true;
-  lastzoom=Zoom;
-}
 
 void zoom(int x, int y)
 {
@@ -1026,8 +1050,8 @@ void zoom(int x, int y)
     }
   }
 }
-  
-void mousewheel(int wheel, int direction, int x, int y) 
+
+void mousewheel(int wheel, int direction, int x, int y)
 {
   double zoomFactor=getSetting<double>("zoomfactor");
   if(zoomFactor > 0.0) {
@@ -1044,17 +1068,15 @@ void mousewheel(int wheel, int direction, int x, int y)
 struct arcball {
   double angle;
   triple axis;
-  
+
   arcball(double x0, double y0, double x, double y) {
     triple v0=norm(x0,y0);
     triple v1=norm(x,y);
     double Dot=dot(v0,v1);
-    if(Dot > 1.0) Dot=1.0;
-    else if(Dot < -1.0) Dot=-1.0;
-    angle=acos(Dot);
+    angle=Dot > 1.0 ? 0.0 : Dot < -1.0 ? pi : acos(Dot);
     axis=unit(cross(v0,v1));
   }
-  
+
   triple norm(double x, double y) {
     double norm=hypot(x,y);
     if(norm > 1.0) {
@@ -1086,13 +1108,13 @@ void rotate(int x, int y)
     update();
   }
 }
-  
-double Degrees(int x, int y) 
+
+double Degrees(int x, int y)
 {
   return atan2(0.5*Height-y-Y,x-0.5*Width-X)*degrees;
 }
 
-void rotateX(double step) 
+void rotateX(double step)
 {
   dmat4 tmpRot(1.0);
   tmpRot=glm::rotate(tmpRot,glm::radians(step),dvec3(1,0,0));
@@ -1100,7 +1122,7 @@ void rotateX(double step)
   update();
 }
 
-void rotateY(double step) 
+void rotateY(double step)
 {
   dmat4 tmpRot(1.0);
   tmpRot=glm::rotate(tmpRot,glm::radians(step),dvec3(0,1,0));
@@ -1108,7 +1130,7 @@ void rotateY(double step)
   update();
 }
 
-void rotateZ(double step) 
+void rotateZ(double step)
 {
   dmat4 tmpRot(1.0);
   tmpRot=glm::rotate(tmpRot,glm::radians(step),dvec3(0,0,1));
@@ -1145,7 +1167,7 @@ void rotateZ(int x, int y)
 #define GLUT_WHEEL_DOWN 4
 #endif
 
-string action(int button, int mod) 
+string action(int button, int mod)
 {
   size_t Button;
   size_t nButtons=5;
@@ -1168,7 +1190,7 @@ string action(int button, int mod)
     default:
       Button=nButtons;
   }
-    
+
   size_t Mod;
   size_t nMods=4;
   switch(mod) {
@@ -1187,7 +1209,7 @@ string action(int button, int mod)
     default:
       Mod=nMods;
   }
-    
+
   if(Button < nButtons) {
     array *left=getSetting<array *>("leftbutton");
     array *middle=getSetting<array *>("middlebutton");
@@ -1216,13 +1238,13 @@ void mouse(int button, int state, int x, int y)
     glutMotionFunc(NULL);
     mousewheel(0,1,x,y);
     return;
-  } 
+  }
   if(Action == "zoomout") {
     glutMotionFunc(NULL);
     mousewheel(0,-1,x,y);
     return;
-  }     
-  
+  }
+
   if(state == GLUT_DOWN) {
     if(Action == "rotate") {
       x0=x; y0=y;
@@ -1247,11 +1269,11 @@ void mouse(int button, int state, int x, int y)
       glutMotionFunc(rotateZ);
     }
   } else {
-     glutMotionFunc(NULL);
+    glutMotionFunc(NULL);
   }
 }
 
-double spinstep() 
+double spinstep()
 {
   timeval tv;
   gettimeofday(&tv,NULL);
@@ -1277,14 +1299,14 @@ void zspin()
   rotateZ(spinstep());
 }
 
-void expand() 
+void expand()
 {
   double resizeStep=getSetting<double>("resizestep");
   if(resizeStep > 0.0)
     setsize((int) (Width*resizeStep+0.5),(int) (Height*resizeStep+0.5));
 }
 
-void shrink() 
+void shrink()
 {
   double resizeStep=getSetting<double>("resizestep");
   if(resizeStep > 0.0)
@@ -1292,7 +1314,7 @@ void shrink()
             max((int) (Height/resizeStep+0.5),1));
 }
 
-void spinx() 
+void spinx()
 {
   if(Xspin)
     idle();
@@ -1328,19 +1350,20 @@ void spinz()
 void showCamera()
 {
   projection P=camera();
+  string projection=P.orthographic ? "orthographic(" : "perspective(";
+  string indent(2+projection.length(),' ');
   cout << endl
-       << "currentprojection=" 
-       << (P.orthographic ? "orthographic(" : "perspective(")  << endl
-       << "camera=" << P.camera << "," << endl
-       << "up=" << P.up << "," << endl
-       << "target=" << P.target << "," << endl
-       << "zoom=" << P.zoom;
+       << "currentprojection=" << endl << "  "
+       << projection << "camera=" << P.camera << "," << endl
+       << indent << "up=" << P.up << "," << endl
+       << indent << "target=" << P.target << "," << endl
+       << indent << "zoom=" << P.zoom;
   if(!orthographic)
-    cout << "," << endl << "angle=" << P.angle;
+    cout << "," << endl << indent << "angle=" << P.angle;
   if(P.viewportshift != pair(0.0,0.0))
-    cout << "," << endl << "viewportshift=" << P.viewportshift*Zoom;
+    cout << "," << endl << indent << "viewportshift=" << P.viewportshift*Zoom;
   if(!orthographic)
-    cout << "," << endl << "autoadjust=false";
+    cout << "," << endl << indent << "autoadjust=false";
   cout << ");" << endl;
 }
 
@@ -1407,7 +1430,7 @@ void keyboard(unsigned char key, int x, int y)
       break;
   }
 }
- 
+
 void setosize()
 {
   oldWidth=(int) ceil(oWidth);
@@ -1418,18 +1441,22 @@ void setosize()
 
 void exportHandler(int=0)
 {
-#ifdef HAVE_LIBGLUT  
-  bool offscreen=getSetting<bool>("offscreen");
-  if(!Iconify && !offscreen)
+#ifdef HAVE_LIBGLUT
+#ifndef HAVE_LIBOSMESA
+  if(!Iconify)
     glutShowWindow();
-#endif  
+#endif
+#endif
   readyAfterExport=true;
   Export();
-#ifdef HAVE_LIBGLUT  
-  if(!Iconify && !offscreen)
+
+#ifdef HAVE_LIBGLUT
+#ifndef HAVE_LIBOSMESA
+  if(!Iconify)
     glutHideWindow();
-#endif  
   glutDisplayFunc(nodisplay);
+#endif
+#endif
 }
 
 static bool glinitialize=true;
@@ -1437,9 +1464,9 @@ static bool glinitialize=true;
 projection camera(bool user)
 {
   if(glinitialize) return projection();
-                   
+
   camp::Triple vCamera,vUp,vTarget;
-  
+
   double cz=0.5*(zmin+zmax);
 
   double *Rotate=value_ptr(drotateMat);
@@ -1475,14 +1502,14 @@ projection camera(bool user)
       vTarget[i]=R3-cx*R0-cy*R1;
     }
   }
-  
+
   return projection(orthographic,vCamera,vUp,vTarget,Zoom,
                     2.0*atan(tan(0.5*Angle)/Zoom)/radians,
                     pair(X/Width+Shift.getx(),
                          Y/Height+Shift.gety()));
 }
 
-void init() 
+void init()
 {
 #ifdef HAVE_LIBGLUT
   mem::vector<string> cmd;
@@ -1495,7 +1522,7 @@ void init()
 
 #ifndef __APPLE__
   glutInitContextProfile(GLUT_CORE_PROFILE);
-#endif  
+#endif
 
   glutInit(&argc,argv);
   screenWidth=glutGet(GLUT_SCREEN_WIDTH);
@@ -1507,7 +1534,7 @@ void init_osmesa()
 {
 #ifdef HAVE_LIBOSMESA
   // create context and buffer
-  if(settings::verbose > 1) 
+  if(settings::verbose > 1)
     cout << "Allocating osmesa_buffer of size " << screenWidth << "x"
          << screenHeight << "x4x" << sizeof(GLubyte) << endl;
   osmesa_buffer=new unsigned char[screenWidth*screenHeight*4*sizeof(GLubyte)];
@@ -1521,7 +1548,7 @@ void init_osmesa()
     cerr << "OSMesaCreateContext failed." << endl;
     exit(-1);
   }
-  
+
   if(!OSMesaMakeCurrent(ctx,osmesa_buffer,GL_UNSIGNED_BYTE,
                         screenWidth,screenHeight )) {
     cerr << "OSMesaMakeCurrent failed." << endl;
@@ -1532,10 +1559,10 @@ void init_osmesa()
   glGetIntegerv(GL_DEPTH_BITS,&z);
   glGetIntegerv(GL_STENCIL_BITS,&s);
   glGetIntegerv(GL_ACCUM_RED_BITS,&a);
-  if(settings::verbose > 1) 
-    cout << "Offscreen context settings: Depth=" << z << " Stencil=" << s 
+  if(settings::verbose > 1)
+    cout << "Offscreen context settings: Depth=" << z << " Stencil=" << s
          << " Accum=" << a << endl;
-  
+
   if(z <= 0) {
     cerr << "Error initializing offscreen context: Depth=" << z << endl;
     exit(-1);
@@ -1555,17 +1582,14 @@ void glrender(const string& prefix, const picture *pic, const string& format,
 {
   Iconify=getSetting<bool>("iconify");
 
-  width=max(width,1.0);
-  height=max(height,1.0);
-  
   if(zoom == 0.0) zoom=1.0;
-  
+
   Prefix=prefix;
   Picture=pic;
   Format=format;
-  
+
   nlights0=nlights=nlightsin;
-  
+
   Lights=lights;
   Diffuse=diffuse;
   Specular=specular;
@@ -1576,21 +1600,21 @@ void glrender(const string& prefix, const picture *pic, const string& format,
   Shift=shift/zoom;
   Margin=margin;
   Background=background;
-  
+
   Xmin=m.getx();
   Xmax=M.getx();
   Ymin=m.gety();
   Ymax=M.gety();
   zmin=m.getz();
   zmax=M.getz();
-  
+
   orthographic=Angle == 0.0;
   H=orthographic ? 0.0 : -tan(0.5*Angle)*zmax;
-  
+
   ignorezoom=false;
   Mode=0;
   Xfactor=Yfactor=1.0;
-  
+
   pair maxtile=getSetting<pair>("maxtile");
   maxTileWidth=(int) maxtile.getx();
   maxTileHeight=(int) maxtile.gety();
@@ -1598,33 +1622,36 @@ void glrender(const string& prefix, const picture *pic, const string& format,
   if(maxTileHeight <= 0) maxTileHeight=768;
 
   bool webgl=Format == "html";
-  
-#ifdef HAVE_GL  
+
+#ifdef HAVE_GL
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
   static bool initializedView=false;
-#endif  
-  
-  bool offscreen=getSetting<bool>("offscreen");
-  if(offscreen && !webgl) {
+#endif
+
+#ifdef HAVE_LIBOSMESA
+  if(!webgl) {
     screenWidth=maxTileWidth;
     screenHeight=maxTileHeight;
 
     static bool osmesa_initialized=false;
     if(!osmesa_initialized) {
       osmesa_initialized=true;
+      fpu_trap(false); // Work around FE_INVALID.
       init_osmesa();
+      fpu_trap(settings::trap());
     }
   }
-  
+#else
   if(glinitialize) {
     if(!webgl) init();
     Fitscreen=1;
   }
+#endif
 #endif
 
   static bool initialized=false;
 
-  if(!(initialized && (interact::interactive || 
+  if(!(initialized && (interact::interactive ||
                        getSetting<bool>("animating")))) {
     antialias=getSetting<Int>("antialias") > 1;
     double expand;
@@ -1636,155 +1663,166 @@ void glrender(const string& prefix, const picture *pic, const string& format,
         expand *= (Format.empty() || Format == "eps" || Format == "pdf")                 ? -2.0 : -1.0;
       if(antialias) expand *= 2.0;
     }
-  
+
     oWidth=width;
     oHeight=height;
     Aspect=width/height;
-  
+
     // Force a hard viewport limit to work around direct rendering bugs.
     // Alternatively, one can use -glOptions=-indirect (with a performance
     // penalty).
     pair maxViewport=getSetting<pair>("maxviewport");
-    maxWidth=(int) ceil(maxViewport.getx());
-    maxHeight=(int) ceil(maxViewport.gety());
+    int maxWidth=maxViewport.getx() > 0 ? (int) ceil(maxViewport.getx()) :
+      screenWidth;
+    int maxHeight=maxViewport.gety() > 0 ? (int) ceil(maxViewport.gety()) :
+      screenHeight;
     if(maxWidth <= 0) maxWidth=max(maxHeight,2);
     if(maxHeight <= 0) maxHeight=max(maxWidth,2);
-    
+
+    if(screenWidth <= 0) screenWidth=maxWidth;
+    else screenWidth=min(screenWidth,maxWidth);
+    if(screenHeight <= 0) screenHeight=maxHeight;
+    else screenHeight=min(screenHeight,maxHeight);
+
     fullWidth=(int) ceil(expand*width);
     fullHeight=(int) ceil(expand*height);
-  
+
     if(webgl) {
       Width=fullWidth;
       Height=fullHeight;
     } else {
-      if(screenWidth <= 0) screenWidth=maxWidth;
-      else screenWidth=min(screenWidth,maxWidth);
-      if(screenHeight <= 0) screenHeight=maxHeight;
-      else screenHeight=min(screenHeight,maxHeight);
-
       Width=min(fullWidth,screenWidth);
       Height=min(fullHeight,screenHeight);
-  
-      if(Width > Height*Aspect) 
+
+      if(Width > Height*Aspect)
         Width=min((int) (ceil(Height*Aspect)),screenWidth);
-      else 
+      else
         Height=min((int) (ceil(Width/Aspect)),screenHeight);
     }
-  
+
     home(webgl);
     setProjection();
     if(webgl) return;
-    
+
     ArcballFactor=1+8.0*hypot(Margin.getx(),Margin.gety())/hypot(Width,Height);
 
-#ifdef HAVE_GL    
+#ifdef HAVE_GL
     for(int i=0; i < 16; ++i)
       T[i]=t[i];
-  
-    remesh=true;
+
     Aspect=((double) Width)/Height;
 
     if(maxTileWidth <= 0) maxTileWidth=screenWidth;
     if(maxTileHeight <= 0) maxTileHeight=screenHeight;
-#ifdef HAVE_LIBGLUT    
+#ifdef HAVE_LIBGLUT
     setosize();
 #endif
-    
+
     if(View && settings::verbose > 1)
       cout << "Rendering " << stripDir(prefix) << " as "
            << Width << "x" << Height << " image" << endl;
-#endif    
+#endif
   }
 
-#ifdef HAVE_GL    
-  bool havewindow=initialized && glthread && !offscreen;
-  
-#ifdef HAVE_LIBGLUT    
+#ifdef HAVE_GL
+  bool havewindow=initialized && glthread;
+
+#ifndef HAVE_LIBOSMESA
+#ifdef HAVE_LIBGLUT
   unsigned int displaymode=GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH;
-#endif  
-  
+#endif
+
 #ifdef __APPLE__
   displaymode |= GLUT_3_2_CORE_PROFILE;
-#endif  
-  
-  camp::clearMaterialBuffer();
-  
-#if defined(MIKTEX) || defined(HAVE_PTHREAD)
-  if(glthread && initializedView && !offscreen) {
-    if(!View)
-      readyAfterExport=queueExport=true;
-#if defined(MIKTEX) && !defined(HAVE_PTHREAD)
-    *MiKTeX::Aymptote::sigusr1 = true;
-#else
-    pthread_kill(mainthread,SIGUSR1);
 #endif
+#endif
+
+  camp::clearMaterialBuffer();
+
+#ifndef HAVE_LIBOSMESA
+
+#if defined(MIKTEX) || defined(HAVE_PTHREAD)
+  if(glthread && initializedView) {
+    if(View) {
+#ifdef __MSDOS__ // Signals are unreliable in MSWindows
+      glupdate=true;
+#else
+#if defined(MIKTEX) && !defined(HAVE_PTHREAD)
+      *MiKTeX::Aymptote::sigusr1 = true;
+#else
+      pthread_kill(mainthread,SIGUSR1);
+#endif
+#endif
+    } else readyAfterExport=queueExport=true;
     return;
   }
-#endif    
-  
-#ifdef HAVE_LIBGLUT
-  if(!offscreen) {
-    if(View) {
-      int x,y;
-      if(havewindow)
-        glutDestroyWindow(window);
-    
-      windowposition(x,y);
-      glutInitWindowPosition(x,y);
-      glutInitWindowSize(1,1);
-      Int multisample=getSetting<Int>("multisample");
-      if(multisample <= 1) multisample=0;
-      if(multisample)
-        displaymode |= GLUT_MULTISAMPLE;
-      glutInitDisplayMode(displaymode);
+#endif
 
-      int samples;
+#ifdef HAVE_LIBGLUT
+  if(View) {
+    int x,y;
+    if(havewindow)
+      glutDestroyWindow(window);
+
+    windowposition(x,y);
+    glutInitWindowPosition(x,y);
+    glutInitWindowSize(1,1);
+    Int multisample=getSetting<Int>("multisample");
+    if(multisample <= 1) multisample=0;
+    if(multisample)
+      displaymode |= GLUT_MULTISAMPLE;
+    glutInitDisplayMode(displaymode);
+
+    int samples;
 
 #ifdef FREEGLUT
 #ifdef GLUT_INIT_MAJOR_VERSION
-      while(true) {
+    while(true) {
 #if defined(MIKTEX) && !defined(HAVE_PTHREAD)
         MiKTeX::Aymptote::RequestHandler();
 #endif
-        if(multisample > 0)
-          glutSetOption(GLUT_MULTISAMPLE,multisample);
-#endif      
-#endif      
-        string title=string(settings::PROGRAM)+": "+prefix;
-        window=glutCreateWindow(title.c_str());
+      if(multisample > 0)
+        glutSetOption(GLUT_MULTISAMPLE,multisample);
+#endif
+#endif
+      string title=string(settings::PROGRAM)+": "+prefix;
+      window=glutCreateWindow(title.c_str());
 
-        GLint samplebuf[1];
-        glGetIntegerv(GL_SAMPLES,samplebuf);
-        samples=samplebuf[0];
+      GLint samplebuf[1];
+      glGetIntegerv(GL_SAMPLES,samplebuf);
+      samples=samplebuf[0];
 #ifdef FREEGLUT
 #ifdef GLUT_INIT_MAJOR_VERSION
-        if(samples < multisample) {
-          multisample=floorpow2(multisample-1);
-          if(multisample > 1) {
-            glutReshapeWindow(1,1);
-            glutDisplayFunc(destroywindow);
-            glutShowWindow();
-            glutMainLoopEvent();
-            continue;
-          }
+      if(samples < multisample) {
+        multisample=floorpow2(multisample-1);
+        if(multisample > 1) {
+          glutReshapeWindow(1,1);
+          glutDisplayFunc(destroywindow);
+          glutShowWindow();
+          glutMainLoopEvent();
+          continue;
         }
-        break;
       }
-#endif      
-#endif      
-      if(settings::verbose > 1 && samples > 1)
-        cout << "Multisampling enabled with sample width " << samples
-             << endl;
-      glutDisplayFunc(display);
-      glutShowWindow();
-    } else if(!havewindow) {
-      glutInitWindowSize(maxTileWidth,maxTileHeight);
-      glutInitDisplayMode(displaymode);
-      window=glutCreateWindow("");
-      glutHideWindow();
+      break;
     }
+#endif
+#endif
+    if(settings::verbose > 1 && samples > 1)
+      cout << "Multisampling enabled with sample width " << samples
+           << endl;
+    glutDisplayFunc(display);
+    glutShowWindow();
+  } else if(!havewindow) {
+    glutInitWindowSize(maxTileWidth,maxTileHeight);
+    glutInitDisplayMode(displaymode);
+    fpu_trap(false); // Work around FE_INVALID in Gallium
+    window=glutCreateWindow("");
+    fpu_trap(settings::trap());
+    glutHideWindow();
   }
 #endif // HAVE_LIBGLUT
+#endif // HAVE_LIBOSMESA
+
   initialized=true;
 
   GLint val;
@@ -1800,24 +1838,25 @@ void glrender(const string& prefix, const picture *pic, const string& format,
       cerr << "GLEW initialization error." << endl;
       exit(-1);
     }
-    
+
     initShaders();
     setBuffers();
   }
-  
+
   glClearColor(Background[0],Background[1],Background[2],Background[3]);
-    
+
 #ifdef HAVE_LIBGLUT
-  if(!offscreen) {
-    Animate=getSetting<bool>("autoplay") && glthread;
-  
-    if(View) {
-      if(!getSetting<bool>("fitscreen"))
-        Fitscreen=0;
-      fitscreen();
-      setosize();
-    }
+#ifndef HAVE_LIBOSMESA
+  Animate=getSetting<bool>("autoplay") && glthread;
+
+  if(View) {
+    if(!getSetting<bool>("fitscreen"))
+      Fitscreen=0;
+    firstFit=true;
+    fitscreen();
+    setosize();
   }
+#endif
 #endif
 
   glEnable(GL_BLEND);
@@ -1826,28 +1865,37 @@ void glrender(const string& prefix, const picture *pic, const string& format,
 
   glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
   mode();
-  
-  if(View && !offscreen) {
+
+#ifdef HAVE_LIBOSMESA
+  View=false;
+#endif
+
+  if(View) {
 #ifdef HAVE_LIBGLUT
 #if defined(MIKTEX) || defined(HAVE_PTHREAD)
     initializedView=true;
-#endif    
+#endif
     glutReshapeFunc(reshape);
     glutKeyboardFunc(keyboard);
     glutMouseFunc(mouse);
     glutDisplayFunc(display);
 
+#ifdef __MSDOS__
+    if(glthread && interact::interactive)
+      poll(0);
+#endif
+
     glutMainLoop();
 #endif // HAVE_LIBGLUT
   } else {
-    if(glthread && !offscreen) {
+    if(glthread) {
       if(havewindow) {
         readyAfterExport=true;
 #ifdef HAVE_PTHREAD
         pthread_kill(mainthread,SIGUSR1);
 #elif defined(MIKTEX)
         *MiKTeX::Aymptote::sigusr1 = true;
-#endif    
+#endif
       } else {
         initialized=true;
         readyAfterExport=true;
@@ -1863,8 +1911,8 @@ void glrender(const string& prefix, const picture *pic, const string& format,
       quit();
     }
   }
-  
-#endif /* HAVE_GL */  
+
+#endif /* HAVE_GL */
 }
 
 } // namespace gl
@@ -1879,34 +1927,39 @@ string getLightIndex(size_t const& index, string const& fieldName) {
   ostringstream buf;
   buf << "lights[" << index << "]." << fieldName;
   return Strdup(buf.str());
-} 
+}
 
 string getCenterIndex(size_t const& index) {
   ostringstream buf;
   buf << "Centers[" << index << "]";
   return Strdup(buf.str());
-} 
+}
 
 template<class T>
-void registerBuffer(const std::vector<T>& buffervector, GLuint bufferIndex,
-                    GLint type=GL_ARRAY_BUFFER) {
+void registerBuffer(const std::vector<T>& buffervector, GLuint& bufferIndex,
+                    bool copy, GLenum type=GL_ARRAY_BUFFER) {
   if(!buffervector.empty()) {
+    if(bufferIndex == 0) {
+      glGenBuffers(1,&bufferIndex);
+      copy=true;
+    }
     glBindBuffer(type,bufferIndex);
-    glBufferData(type,buffervector.size()*sizeof(T),
-                 buffervector.data(),GL_STATIC_DRAW);
+    if(copy)
+      glBufferData(type,buffervector.size()*sizeof(T),
+                   buffervector.data(),GL_STATIC_DRAW);
   }
 }
 
-void setUniforms(const vertexBuffer& data, GLint shader)
+void setUniforms(vertexBuffer& data, GLint shader)
 {
   bool normal=shader != pixelShader;
-    
+
   if(shader != gl::lastshader) {
     glUseProgram(shader);
     gl::lastshader=shader;
-  
+
     glUniform1i(glGetUniformLocation(shader,"nlights"),gl::nlights);
-  
+
     for(size_t i=0; i < gl::nlights; ++i) {
       triple Lighti=gl::Lights[i];
       size_t i4=4*i;
@@ -1923,7 +1976,7 @@ void setUniforms(const vertexBuffer& data, GLint shader)
 
 #if HAVE_LIBOPENIMAGEIO
     // textures
-    if (settings::getSetting<bool>("envmap")) { 
+    if (settings::getSetting<bool>("envmap")) {
       glActiveTexture(GL_TEXTURE1);
       glBindBuffer(GL_TEXTURE_2D, gl::envMapBuf);
       glUniform1i(glGetUniformLocation(shader, "environmentMap"), 1);
@@ -1931,16 +1984,17 @@ void setUniforms(const vertexBuffer& data, GLint shader)
     }
 #endif
   }
-  
+
   GLuint binding=0;
   GLint blockindex=glGetUniformBlockIndex(shader,"MaterialBuffer");
   glUniformBlockBinding(shader,blockindex,binding);
-  registerBuffer(data.materials,gl::ubo,GL_UNIFORM_BUFFER);
-  glBindBufferBase(GL_UNIFORM_BUFFER,binding,gl::ubo);
-  
+  bool copy=gl::remesh || data.partial || !data.rendered;
+  registerBuffer(data.materials,data.materialsBuffer,copy,GL_UNIFORM_BUFFER);
+  glBindBufferBase(GL_UNIFORM_BUFFER,binding,data.materialsBuffer);
+
   glUniformMatrix4fv(glGetUniformLocation(shader,"projViewMat"),1,GL_FALSE,
                      value_ptr(gl::projViewMat));
-  
+
   glUniformMatrix4fv(glGetUniformLocation(shader,"viewMat"),1,GL_FALSE,
                      value_ptr(gl::viewMat));
   if(normal)
@@ -1951,30 +2005,30 @@ void setUniforms(const vertexBuffer& data, GLint shader)
 void drawBuffer(vertexBuffer& data, GLint shader)
 {
   if(data.indices.empty()) return;
-  
+
   bool normal=shader != pixelShader;
   bool color=shader == colorShader || shader == transparentShader;
-  
+
   const size_t size=sizeof(GLfloat);
   const size_t intsize=sizeof(GLint);
   const size_t bytestride=color ? sizeof(VertexData) :
     (normal ? sizeof(vertexData) : sizeof(vertexData0));
 
-  if(color) registerBuffer(data.Vertices,attributeBuffer);
-  else if(normal) registerBuffer(data.vertices,attributeBuffer);
-  else registerBuffer(data.vertices0,attributeBuffer);
-  
-  registerBuffer(data.indices,indicesBuffer,GL_ELEMENT_ARRAY_BUFFER);
-  
-  glBindBuffer(GL_ARRAY_BUFFER,attributeBuffer);
-  glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,indicesBuffer);
+  bool copy=gl::remesh || data.partial || !data.rendered;
+  if(color) registerBuffer(data.Vertices,data.VerticesBuffer,copy);
+  else if(normal) registerBuffer(data.vertices,data.verticesBuffer,copy);
+  else registerBuffer(data.vertices0,data.vertices0Buffer,copy);
+
+  registerBuffer(data.indices,data.indicesBuffer,copy,GL_ELEMENT_ARRAY_BUFFER);
 
   camp::setUniforms(data,shader);
 
+  data.rendered=true;
+
   glVertexAttribPointer(positionAttrib,3,GL_FLOAT,GL_FALSE,bytestride,
                         (void *) 0);
   glEnableVertexAttribArray(positionAttrib);
-    
+
   if(normal && gl::Nlights > 0) {
     glVertexAttribPointer(normalAttrib,3,GL_FLOAT,GL_FALSE,bytestride,
                           (void *) (3*size));
@@ -1984,8 +2038,8 @@ void drawBuffer(vertexBuffer& data, GLint shader)
                           (void *) (3*size));
     glEnableVertexAttribArray(widthAttrib);
   }
-    
-  glVertexAttribIPointer(materialAttrib,1,GL_INT,bytestride, 
+
+  glVertexAttribIPointer(materialAttrib,1,GL_INT,bytestride,
                          (void *) ((normal ? 6 : 4)*size));
   glEnableVertexAttribArray(materialAttrib);
 
@@ -1994,20 +2048,20 @@ void drawBuffer(vertexBuffer& data, GLint shader)
                           (void *) (6*size+intsize));
     glEnableVertexAttribArray(colorAttrib);
   }
-  
+
   glDrawElements(data.type,data.indices.size(),GL_UNSIGNED_INT,(void *) 0);
 
   glDisableVertexAttribArray(positionAttrib);
   if(normal && gl::Nlights > 0)
     glDisableVertexAttribArray(normalAttrib);
   if(!normal)
-   glDisableVertexAttribArray(widthAttrib);
+    glDisableVertexAttribArray(widthAttrib);
   glDisableVertexAttribArray(materialAttrib);
   if(color)
     glDisableVertexAttribArray(colorAttrib);
-  
+
   glBindBuffer(GL_UNIFORM_BUFFER,0);
-  
+
   glBindBuffer(GL_ARRAY_BUFFER,0);
   glBindBuffer(GL_ELEMENT_ARRAY_BUFFER,0);
 }
@@ -2039,6 +2093,7 @@ void drawColor()
 void drawTriangle()
 {
   drawBuffer(triangleData,transparentShader);
+  triangleData.rendered=false; // Force copying of sorted triangles to GPU.
   triangleData.clear();
 }
 
@@ -2047,6 +2102,7 @@ void drawTransparent()
   sortTriangles();
   glDepthMask(GL_FALSE); // Enable transparency
   drawBuffer(transparentData,transparentShader);
+  transparentData.rendered=false; // Force copying of sorted triangles to GPU.
   glDepthMask(GL_TRUE); // Disable transparency
   transparentData.clear();
 }
@@ -2067,14 +2123,23 @@ void clearMaterialBuffer()
   material.reserve(nmaterials);
   materialMap.clear();
   materialIndex=0;
+
+  material0Data.partial=false;
+  material1Data.partial=false;
+  materialData.partial=false;
+  colorData.partial=false;
+  triangleData.partial=false;
+  transparentData.partial=false;
 }
 
 void setMaterial(vertexBuffer& data, draw_t *draw)
 {
   if(materialIndex >= data.materialTable.size() ||
      data.materialTable[materialIndex] == -1) {
-    if(data.materials.size() >= Maxmaterials)
+    if(data.materials.size() >= Maxmaterials) {
+      data.partial=true;
       (*draw)();
+    }
     size_t size0=data.materialTable.size();
     data.materialTable.resize(materialIndex+1);
     for(size_t i=size0; i < materialIndex; ++i)
