@@ -20,7 +20,7 @@ using vm::read;
 
 namespace camp {
 
-texfile::texfile(const string& texname, const bbox& box, bool pipe) 
+texfile::texfile(const string& texname, const bbox& box, bool pipe)
   : box(box)
 {
   texengine=getSetting<string>("tex");
@@ -39,11 +39,11 @@ texfile::texfile(const string& texname, const bbox& box, bool pipe)
 texfile::~texfile()
 {
   if(out) {
-    delete out;  
+    delete out;
     out=NULL;
   }
 }
-  
+
 void texfile::miniprologue()
 {
   texpreamble(*out,processData().TeXpreamble,true);
@@ -68,41 +68,36 @@ void texfile::prologue()
     texpreamble(*outpreamble,processData().TeXpreamble,false,false);
     outpreamble->close();
   }
-  
+
   texdefines(*out,processData().TeXpreamble,false);
   double width=box.right-box.left;
   double height=box.top-box.bottom;
   if(!inlinetex) {
     if(settings::context(texengine)) {
-      *out << "\\definepapersize[asy][width=" << width << "bp,height=" 
+      *out << "\\definepapersize[asy][width=" << width << "bp,height="
            << height << "bp]" << newl
            << "\\setuppapersize[asy][asy]" << newl;
     } else if(settings::pdf(texengine)) {
-      double voffset=0.0;
-      if(settings::latex(texengine)) {
-        if(height < 12.0) voffset=height-12.0;
-      } else if(height < 10.0) voffset=height-10.0;
-
-      if(width > 0) 
+      if(width > 0)
         *out << "\\pdfpagewidth=" << width << "bp" << newl;
       *out << "\\ifx\\pdfhorigin\\undefined" << newl
            << "\\hoffset=-1in" << newl
-           << "\\voffset=" << voffset-72.0 << "bp" << newl;
+           << "\\voffset=-1in" << newl;
       if(height > 0)
-        *out << "\\pdfpageheight=" << height << "bp" 
+        *out << "\\pdfpageheight=" << height << "bp"
              << newl;
       *out << "\\else" << newl
            << "\\pdfhorigin=0bp" << newl
-           << "\\pdfvorigin=" << voffset << "bp" << newl;
+           << "\\pdfvorigin=0bp" << newl;
       if(height > 0)
         *out << "\\pdfpageheight=" << height << "bp" << newl;
       *out << "\\fi" << newl;
     }
   }
-  
+
   if(settings::xe(texengine) && !inlinetex)
     *out << "\\usepackage{everypage}%" << newl;
-  
+
   if(settings::latex(texengine)) {
     *out << "\\setlength{\\unitlength}{1pt}%" << newl;
     if(!inlinetex) {
@@ -141,10 +136,10 @@ void texfile::prologue()
       }
     }
   }
-  
+
   beginpage();
 }
-    
+
 void texfile::beginlayer(const string& psname, bool postscript)
 {
   if(box.right > box.left && box.top > box.bottom) {
@@ -162,7 +157,7 @@ void texfile::beginlayer(const string& psname, bool postscript)
         } else {
           if(!pdf) name=psname;
         }
-        
+
         if(!pdf)
           *out << "[bb=" << box.left << " " << box.bottom << " "
                << box.right << " " << box.top << "]";
@@ -171,7 +166,7 @@ void texfile::beginlayer(const string& psname, bool postscript)
       if(!inlinetex)
         *out << "\\kern " << (box.left-box.right)*ps2tex << "pt%" << newl;
     } else {
-      *out << "\\leavevmode\\vbox to " << (box.top-box.bottom)*ps2tex 
+      *out << "\\leavevmode\\vbox to " << (box.top-box.bottom)*ps2tex
            << "pt{}%" << newl;
       if(inlinetex)
         *out << "\\kern " << (box.right-box.left)*ps2tex << "pt%" << newl;
@@ -192,52 +187,52 @@ void texfile::writeshifted(path p, bool newPath)
 
 void texfile::setlatexcolor(pen p)
 {
-  if(p.cmyk() && (!lastpen.cmyk() || 
-                  (p.cyan() != lastpen.cyan() || 
-                   p.magenta() != lastpen.magenta() || 
+  if(p.cmyk() && (!lastpen.cmyk() ||
+                  (p.cyan() != lastpen.cyan() ||
+                   p.magenta() != lastpen.magenta() ||
                    p.yellow() != lastpen.yellow() ||
                    p.black() != lastpen.black()))) {
-    *out << "\\definecolor{ASYcolor}{cmyk}{" 
-         << p.cyan() << "," << p.magenta() << "," << p.yellow() << "," 
+    *out << "\\definecolor{ASYcolor}{cmyk}{"
+         << p.cyan() << "," << p.magenta() << "," << p.yellow() << ","
          << p.black() << "}\\color{ASYcolor}%" << newl;
   } else if(p.rgb() && (!lastpen.rgb() ||
                         (p.red() != lastpen.red() ||
-                         p.green() != lastpen.green() || 
+                         p.green() != lastpen.green() ||
                          p.blue() != lastpen.blue()))) {
-    *out << "\\definecolor{ASYcolor}{rgb}{" 
+    *out << "\\definecolor{ASYcolor}{rgb}{"
          << p.red() << "," << p.green() << "," << p.blue()
          << "}\\color{ASYcolor}%" << newl;
-  } else if(p.grayscale() && (!lastpen.grayscale() || 
+  } else if(p.grayscale() && (!lastpen.grayscale() ||
                               p.gray() != lastpen.gray())) {
-    *out << "\\definecolor{ASYcolor}{gray}{" 
+    *out << "\\definecolor{ASYcolor}{gray}{"
          << p.gray()
          << "}\\color{ASYcolor}%" << newl;
   }
 }
-  
+
 void texfile::setfont(pen p)
 {
   bool latex=settings::latex(texengine);
-  
+
   if(latex) setlatexfont(*out,p,lastpen);
   settexfont(*out,p,lastpen,latex);
-  
+
   lastpen.setfont(p);
 }
-  
+
 void texfile::setpen(pen p)
 {
   bool latex=settings::latex(texengine);
-  
+
   p.convert();
   if(p == lastpen) return;
 
   if(latex) setlatexcolor(p);
   else setcolor(p,settings::beginspecial(texengine),settings::endspecial());
-  
+
   setfont(p);
 }
-   
+
 void texfile::beginpicture(const bbox& b)
 {
   verbatim(settings::beginpicture(texengine));
@@ -252,7 +247,7 @@ void texfile::beginpicture(const bbox& b)
   }
   verbatimline("%");
 }
-  
+
 void texfile::endpicture(const bbox& b)
 {
   verbatimline(settings::endpicture(texengine));
@@ -261,7 +256,7 @@ void texfile::endpicture(const bbox& b)
   write(-width*ps2tex);
   verbatimline("pt%");
 }
-  
+
 void texfile::gsave(bool)
 {
   *out << settings::beginspecial(texengine);
@@ -276,41 +271,41 @@ void texfile::grestore(bool)
   *out << settings::endspecial() << newl;
 }
 
-void texfile::beginspecial() 
+void texfile::beginspecial()
 {
   *out << settings::beginspecial(texengine);
 }
-  
-void texfile::endspecial() 
+
+void texfile::endspecial()
 {
   *out << settings::endspecial() << newl;
 }
-  
-void texfile::beginraw() 
+
+void texfile::beginraw()
 {
   *out << "\\ASYraw{" << newl;
 }
-  
-void texfile::endraw() 
+
+void texfile::endraw()
 {
   *out << "}%" << newl;
 }
-  
+
 void texfile::put(const string& label, const transform& T, const pair& z,
                   const pair& align)
 {
   double sign=settings::pdf(texengine) ? 1.0 : -1.0;
 
   if(label.empty()) return;
-  
+
   bool trans=!T.isIdentity();
-  
+
   *out << "\\ASYalign";
   if(trans) *out << "T";
-  *out << "(" << (z.getx()-Hoffset)*ps2tex
-       << "," << (z.gety()-box.bottom)*ps2tex
+  *out << "(" << (z.getx()-hoffset())*ps2tex
+       << "," << (z.gety()-voffset())*ps2tex
        << ")(" << align.getx()
-       << "," << align.gety() 
+       << "," << align.gety()
        << ")";
   if(trans)
     *out << "{" << T.getxx() << " " << sign*T.getyx()
@@ -332,35 +327,52 @@ void texfile::epilogue(bool pipe)
 
 string svgtexfile::nl="{?nl}%\n";
 
-void svgtexfile::beginspecial()
+void svgtexfile::beginspecial(bool def)
 {
   inspecial=true;
   out->unsetf(std::ios::fixed);
   *out << "\\catcode`\\#=11%" << newl
-       << "\\special{dvisvgm:raw" << nl;
+       << "\\special{dvisvgm:raw";
+  if(def)
+    *out << "def";
+  *out << nl;
 }
-    
+
 void svgtexfile::endspecial()
 {
+  if(!inspecial)
+    reportError("endspecial without matching beginspecial");
   inspecial=false;
   *out << "}\\catcode`\\#=6%" << newl;
   out->setf(std::ios::fixed);
 }
-  
-void svgtexfile::begintransform()
+
+void svgtexfile::transform()
 {
   bbox b=box;
   b.left=-Hoffset;
   b=svgbbox(b);
-  *out << "<g transform='matrix(" << tex2ps << " 0 0 " << tex2ps <<" "
-       << b.left << " " << b.top << ")'>" << nl;
+  *out << "transform='matrix(" << tex2ps << " 0 0 " << tex2ps <<" "
+       << b.left << " " << b.top << ")'";
 }
-    
+
+void svgtexfile::begintransform()
+{
+  if(clipstack.size() > 0) {
+    *out << "</g><g ";
+    clippath();
+    *out << ">" << nl;
+  }
+  *out << "<g ";
+  transform();
+  *out << ">" << nl;
+}
+
 void svgtexfile::endtransform()
 {
   *out << "</g>";
 }
-  
+
 void svgtexfile::gsave(bool)
 {
   if(clipstack.size() < 1)
@@ -370,7 +382,7 @@ void svgtexfile::gsave(bool)
   *out << "\\special{dvisvgm:raw <g>}%" << newl;
   pens.push(lastpen);
 }
-  
+
 void svgtexfile::grestore(bool)
 {
   if(pens.size() < 1 || clipstack.size() < 1)
@@ -386,71 +398,64 @@ void svgtexfile::clippath()
   if(clipstack.size() > 0) {
     size_t count=clipstack.top();
     if(count > 0)
-      *out << "clip-path='url(#clip" << count << ")' ";
+      *out << "clip-path='url(#Clip" << count << ")' ";
   }
 }
-  
+
 void svgtexfile::beginpath()
 {
   *out << "<path ";
-  clippath();
   *out << "d='";
 }
-  
+
 void svgtexfile::endpath()
 {
   *out << "/>" << nl;
 }
-  
+
 void svgtexfile::dot(path p, pen q, bool newPath)
 {
   beginspecial();
   begintransform();
   *out << "<circle ";
-  clippath();
   pair z=p.point((Int) 0);
-  *out << "cx='" << z.getx()*ps2tex
-       << "' cy='" << -z.gety()*ps2tex
+  *out << "cx='" << (z.getx()-offset.getx())*ps2tex
+       << "' cy='" << (-z.gety()+offset.gety())*ps2tex
        << "' r='" << 0.5*q.width()*ps2tex;
 }
 
 void svgtexfile::beginclip()
 {
-  beginspecial();
-  begintransform();
+  beginspecial(true);
   *out << "<clipPath ";
   clippath();
   ++clipcount;
-  *out << "id='clip" << clipcount << "'>" << nl;
-  beginpath();
+  *out << "id='Clip" << clipcount << "'>" << nl;
+  *out << "<path ";
+  transform();
+  *out << " d='";
   if(clipstack.size() > 0)
     clipstack.pop();
   clipstack.push(clipcount);
 }
-  
-void svgtexfile::endclip0(const pen &p) 
+
+void svgtexfile::endclip(const pen &p)
 {
   *out << "'";
   fillrule(p,"clip");
   endpath();
   *out << "</clipPath>" << nl;
-}
-
-void svgtexfile::endclip(const pen &p) 
-{
-  endclip0(p);
-  endtransform();
   endspecial();
 }
 
 void svgtexfile::fillrule(const pen& p, const string& type)
 {
   if(p.Fillrule() != lastpen.Fillrule())
-    *out << " " << type << "-rule='" << 
+    *out << " " << type << "-rule='" <<
       (p.evenodd() ? "evenodd" : "nonzero") << "'";
   lastpen.setfillrule(p);
 }
-   
+
 void svgtexfile::color(const pen &p, const string& type)
 {
   *out << "' " << type << "='#" << rgbhex(p) << "'";
@@ -472,19 +477,19 @@ void svgtexfile::properties(const pen& p)
 {
   if(p.cap() != lastpen.cap())
     *out << " stroke-linecap='" << PSCap[p.cap()] << "'";
-    
+
   if(p.join() != lastpen.join())
     *out << " stroke-linejoin='" << Join[p.join()] << "'";
-  
+
   if(p.miter() != lastpen.miter())
     *out << " stroke-miterlimit='" << p.miter()*ps2tex << "'";
-    
+
   if(p.width() != lastpen.width())
     *out << " stroke-width='" << p.width()*ps2tex << "'";
-  
+
   const LineType *linetype=p.linetype();
   const LineType *lastlinetype=lastpen.linetype();
-  
+
   if(!(linetype->pattern == lastlinetype->pattern)) {
     size_t n=linetype->pattern.size();
     if(n > 0) {
@@ -495,26 +500,26 @@ void svgtexfile::properties(const pen& p)
       *out << "'";
     }
   }
-  
+
   if(linetype->offset != lastlinetype->offset)
     *out << " stroke-dashoffset='" << linetype->offset*ps2tex << "'";
-  
+
   lastpen=p;
 }
-  
+
 void svgtexfile::stroke(const pen &p, bool dot)
 {
-  if(dot) 
+  if(dot)
     color(p,"fill");
   else {
     color(p,"fill='none' stroke");
-    properties(p);  
+    properties(p);
   }
   endpath();
   endtransform();
   endspecial();
 }
-  
+
 void svgtexfile::strokepath()
 {
   reportWarning("SVG does not support strokepath");
@@ -526,19 +531,19 @@ void svgtexfile::begingradientshade(bool axial, ColorSpace colorspace,
 {
   string type=axial ? "linear" : "radial";
   beginspecial();
+  begintransform();
   *out << "<" << type << "Gradient id='grad" << gradientcount;
   if(axial) {
-    *out << "' x1='" << a.getx()*ps2tex << "' y1='" << -a.gety()*ps2tex
-         << "' x2='" << b.getx()*ps2tex << "' y2='" << -b.gety()*ps2tex;
+    *out << "' x1='" << a.getx()*ps2tex << "' y1='" << (box.top-a.gety())*ps2tex
+         << "' x2='" << b.getx()*ps2tex << "' y2='" << (box.top-b.gety())*ps2tex;
   } else {
-    *out << "' cx='" << b.getx()*ps2tex << "' cy='" << -b.gety()*ps2tex
+    *out << "' cx='" << b.getx()*ps2tex << "' cy='" << (box.top-b.gety())*ps2tex
          << "' r='" << rb*ps2tex;
   }
   *out <<"' gradientUnits='userSpaceOnUse'>" << nl
        << "<stop offset='0' stop-color='#" << rgbhex(pena) << "'/>" << nl
        << "<stop offset='1' stop-color='#" << rgbhex(penb) << "'/>" << nl
        << "</" << type << "Gradient>" << nl;
-  begintransform();
   beginpath();
 }
 
@@ -555,7 +560,7 @@ void svgtexfile::gradientshade(bool axial, ColorSpace colorspace,
   endtransform();
   endspecial();
 }
-  
+
 // Return the point on the line through p and q that is closest to z.
 pair closest(pair p, pair q, pair z)
 {
@@ -565,47 +570,61 @@ pair closest(pair p, pair q, pair z)
 }
 
 void svgtexfile::gouraudshade(const pen& p0, const pair& z0,
-                              const pen& p1, const pair& z1, 
+                              const pen& p1, const pair& z1,
                               const pen& p2, const pair& z2)
 {
   string hex[]={rgbhex(p0),rgbhex(p1),rgbhex(p2)};
-  pair Z[]={z0,z1,z2};
-    
-  *out << "<defs>" << nl
-       << "<filter id='colorAdd'>" << nl
-       << "<feBlend in='SourceGraphic' in2='BackgroundImage'/>" << nl
-       << "</filter>";
+
+  *out << "<defs>" << nl;
+
+  pair Z0=(z0-offset)*ps2tex;
+  pair Z1=(z1-offset)*ps2tex;
+  pair Z2=(z2-offset)*ps2tex;
+
+  pair Z[]={Z0,Z1,Z2};
 
   for(size_t k=0; k < 3; ++k) {
     pair z=Z[k];
     pair opp=closest(Z[(k+1) % 3],Z[(k+2) % 3],z);
-    *out << "<linearGradient id='grad-" << gouraudcount << "-" << k 
+    *out << "<linearGradient id='grad-" << gouraudcount << "-" << k
          << "' gradientUnits='userSpaceOnUse'" << nl
-         << " x1='" << z.getx()*ps2tex << "' y1='" << -z.gety()*ps2tex
-         << "' x2='" << opp.getx()*ps2tex << "' y2='" << -opp.gety()*ps2tex
+         << " x1='" << z.getx() << "' y1='" << -z.gety()
+         << "' x2='" << opp.getx() << "' y2='" << -opp.gety()
          << "'>" << nl
-         << "<stop offset='0' stop-color='#" << hex[k] 
+         << "<stop offset='0' stop-color='#" << hex[k]
          << "' stop-opacity='1'/>" << nl
-         << "<stop offset='1' stop-color='#" << hex[k] 
+         << "<stop offset='1' stop-color='#" << hex[k]
          << "' stop-opacity='0'/>" << nl
          << "</linearGradient>" << nl;
   }
-  *out << "<polygon ";
-  clippath();
-  *out << "id='triangle" << gouraudcount << "' points='"
-       << z0.getx()*ps2tex << "," << -z0.gety()*ps2tex << " "
-       << z1.getx()*ps2tex << "," << -z1.gety()*ps2tex << " "
-       << z2.getx()*ps2tex << "," << -z2.gety()*ps2tex << "'/>" << nl
-       << "</defs>" << nl;
-  *out << "<use xlink:href='#triangle" << gouraudcount
-       << "' fill='url(#grad-" << gouraudcount << "-" 
-       << "0)'/>" << nl
-       << "<use xlink:href='#triangle" << gouraudcount
-       << "' fill='url(#grad-" << gouraudcount << "-" 
-       << "1)' filter='url(#colorAdd)'/>" << nl
-       << "<use xlink:href='#triangle" << gouraudcount
-       << "' fill='url(#grad-" << gouraudcount << "-" 
-       << "2)' filter='url(#colorAdd)'/>" << nl;
+
+  *out << "<polygon points='"
+       << Z0.getx() << "," << -Z0.gety() << " "
+       << Z1.getx() << "," << -Z1.gety() << " "
+       << Z2.getx() << "," << -Z2.gety() << "'"
+       << " id='triangle-" << gouraudcount << "' />" << nl;
+
+  for(unsigned vertex=0; vertex < 3; ++vertex)
+    *out << "<use xlink:href='#triangle-" << gouraudcount
+         << "' fill='url(#grad-" << gouraudcount << "-" << vertex
+         << ")' id='triangle-" << gouraudcount << "-" << vertex << "' />"
+         << nl;
+
+  *out << "<filter id='Gouraud-" << gouraudcount << "'>" << nl;
+
+  for(unsigned vertex=0; vertex < 3; ++vertex)
+    *out << "<feImage xlink:href='#triangle-" << gouraudcount << "-" << vertex
+         << "' result='layer" << vertex << "' x='0' y='0' />" << nl;
+
+  *out << "<feComposite in='layer0' in2='layer1' operator='arithmetic' k1='0' k2='1' k3='1' k4='0' result='temp'/>" << nl
+       << "<feComposite in='temp' in2='layer2' operator='arithmetic' k1='0' k2='1' k3='1' k4='0' result='temp2' />" << nl
+       << "<feComposite in='temp2' in2='SourceGraphic' operator='arithmetic' k1='0' k2='1' k3='1' k4='0'/>" << nl
+       << "</filter>" << nl
+       << "</defs>" << nl
+       << "<rect width='100\\percent' height='100\\percent' fill='none' ";
+  *out << " filter='url(#Gouraud-" << gouraudcount << ")'"
+       << "/>" << nl;
+
   ++gouraudcount;
 }
 
@@ -624,29 +643,31 @@ void svgtexfile::gouraudshade(const pen& pentype,
 {
   size_t size=pens.size();
   if(size == 0) return;
-  
-  endclip0(pentype);
-  
+
+  endclip(pentype);
+  beginspecial();
+  begintransform();
+
   pen *p0=NULL,*p1=NULL,*p2=NULL;
   pair z0,z1,z2;
-  
+
   for(size_t i=0; i < size; i++) {
     Int edge=read<Int>(edges,i);
-    
+
     switch(edge) {
       case 0:
         p0=read<pen *>(pens,i);
         z0=read<pair>(vertices,i);
         ++i;
-	if(i < size) {
-	  p1=read<pen *>(pens,i);
-	  z1=read<pair>(vertices,i);
-	  ++i;
-	  if(i < size) {
-	    p2=read<pen *>(pens,i);
-	    z2=read<pair>(vertices,i);
-	  }
-	}
+        if(i < size) {
+          p1=read<pen *>(pens,i);
+          z1=read<pair>(vertices,i);
+          ++i;
+          if(i < size) {
+            p2=read<pen *>(pens,i);
+            z2=read<pair>(vertices,i);
+          }
+        }
         break;
       case 1:
         p0=read<pen *>(pens,i);
@@ -657,7 +678,7 @@ void svgtexfile::gouraudshade(const pen& pentype,
         z1=read<pair>(vertices,i);
         break;
       default:
-	break;
+        break;
     }
     if(p0 == NULL || p1 == NULL || p2 == NULL)
       reportError("invalid edge flag");
@@ -667,67 +688,4 @@ void svgtexfile::gouraudshade(const pen& pentype,
   endspecial();
 }
 
-void svgtexfile::begintensorshade(const vm::array& pens,
-                                  const vm::array& boundaries,
-                                  const vm::array& z) 
-{
-  beginspecial();
-  *out << "<defs>" << nl;
-
-  path g=read<path>(boundaries,0);
-  pair Z[]={g.point((Int) 0),g.point((Int) 3),g.point((Int) 2),
-            g.point((Int) 1)};
-      
-  array *pi=read<array *>(pens,0);
-  if(checkArray(pi) != 4)
-    reportError("specify 4 pens for each path");
-  string hex[]={rgbhex(read<pen>(pi,0)),rgbhex(read<pen>(pi,3)),
-                rgbhex(read<pen>(pi,2)),rgbhex(read<pen>(pi,1))};
-    
-  *out << "<filter id='colorAdd'>" << nl
-       << "<feBlend in='SourceGraphic' in2='BackgroundImage'/>" << nl
-       << "</filter>";
-
-  pair mean=0.25*(Z[0]+Z[1]+Z[2]+Z[3]);
-  for(size_t k=0; k < 4; ++k) {
-    pair opp=(k % 2 == 0) ? Z[(k+2) % 4] : mean;
-    *out << "<linearGradient id='grad" << tensorcount << "-" << k 
-         << "' gradientUnits='userSpaceOnUse'" << nl
-         << " x1='" << Z[k].getx()*ps2tex << "' y1='" << -Z[k].gety()*ps2tex
-         << "' x2='" << opp.getx()*ps2tex << "' y2='" << -opp.gety()*ps2tex
-         << "'>" << nl
-         << "<stop offset='0' stop-color='#" << hex[k] 
-         << "' stop-opacity='1'/>" << nl
-         << "<stop offset='1' stop-color='#" << hex[k] 
-         << "' stop-opacity='0'/>" << nl
-         << "</linearGradient>" << nl;
-  }
-  beginpath();
-}
-
-void svgtexfile::tensorshade(const pen& pentype, const vm::array& pens,
-                             const vm::array& boundaries, const vm::array& z)
-{
-  *out << "' id='path" << tensorcount << "'";
-  fillrule(pentype);
-  endpath();
-  *out << "</defs></g>" << nl;
-  begintransform();
-  *out << "<use xlink:href='#path" << tensorcount
-       << "' fill='url(#grad" << tensorcount << "-" 
-       << "0)'/>" << nl
-       << "<use xlink:href='#path" << tensorcount
-       << "' fill='url(#grad" << tensorcount << "-" 
-       << "2)' filter='url(#colorAdd)'/>" << nl
-       << "<use xlink:href='#path" << tensorcount
-       << "' fill='url(#grad" << tensorcount << "-" 
-       << "1)' filter='url(#colorAdd)'/>" << nl
-       << "<use xlink:href='#path" << tensorcount 
-       << "' fill='url(#grad" << tensorcount << "-"
-       << "3)' filter='url(#colorAdd)'/>" << nl;
-
-  ++tensorcount;
-  endspecial();
-}
-
 } //namespace camp
